package com.localmatters.flexiq.security;

import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.Key;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;

/**
 * The cipher service base class.
 *
 * @author Daniel Cavalcanti
 */
public abstract class CipherServiceBase
    implements CipherService {
    
    // **********************************************************************
    // Instance variables
    // **********************************************************************
    
    protected KeyGenerator keygen;
    
    protected Key key;
    protected Cipher cipher;
    
    // **********************************************************************
    // Factory methods
    // **********************************************************************
    
    /**
     * Returns a new instance of a cipher. The cipher key is randomly generated.
     * 
     * @param algorithm The cipher algorithm of choice.
     * @throws java.security.GeneralSecurityException If cannot instantiate cipher instance.
     * @return The cipher instance.
     */
    public final static CipherService getInstance(CipherAlgorithmEnum algorithm)
    throws GeneralSecurityException {
        switch (algorithm) {
            case AES :
                return new AESCipherService();
            case Blowfish :
                return new BlowfishCipherService();
            case DES :
                return new DESCipherService();
            case TRIPLE_DES :
                return new TripleDESCipherService();
            default :
                throw new GeneralSecurityException("Cipher algorithm not supported.");
        }
    }
    
    /**
     * Returns a new instance of a cipher.
     * 
     * @param algorithm The cipher algorithm of choice.
     * @param key The key used by the cipher.
     * @throws java.security.GeneralSecurityException If cannot instantiate cipher instance.
     * @return The cipher instance.
     */
    public final static CipherService getInstance(CipherAlgorithmEnum algorithm, Key key)
    throws GeneralSecurityException {
        switch (algorithm) {
            case AES :
                return new AESCipherService(key);
            case Blowfish :
                return new BlowfishCipherService(key);
            case DES :
                return new DESCipherService(key);
            case TRIPLE_DES :
                return new TripleDESCipherService(key);
            default :
                throw new GeneralSecurityException("Cipher algorithm not supported.");
        }
    }
    
    /**
     * Returns a new instance of a cipher.
     * 
     * @param algorithm The cipher algorithm of choice.
     * @param key The key used by the cipher and to be imported.
     * @throws java.security.GeneralSecurityException If cannot instantiate cipher instance.
     * @return The cipher instance.
     */
    public final static CipherService getInstance(CipherAlgorithmEnum algorithm, byte[] key)
    throws GeneralSecurityException {
        CipherService cipher = null;
        switch (algorithm) {
            case AES :
                cipher = new AESCipherService();
                cipher.importKey(key);
                return cipher;
            case Blowfish :
                cipher = new BlowfishCipherService();
                cipher.importKey(key);
                return cipher;
            case DES :
                cipher = new DESCipherService();
                cipher.importKey(key);
                return cipher;
            case TRIPLE_DES :
                cipher = new TripleDESCipherService();
                cipher.importKey(key);
                return cipher;
            default :
                throw new GeneralSecurityException("Cipher algorithm not supported.");
        }
    }
    
    // **********************************************************************
    // SecurityService methods
    // **********************************************************************
    
    public final synchronized Key getKey() {
        return key;
    }
    
    public final synchronized void setKey(Key key)
    throws InvalidKeyException {
        cipher.init(Cipher.ENCRYPT_MODE, key);
        this.key = key;
    }
    
    public final synchronized byte[] exportKey()
    throws GeneralSecurityException {
        return key.getEncoded();
    }
    
    public final synchronized byte[] encrypt(byte[] cleartext)
    throws NullPointerException, GeneralSecurityException {
        cipher.init(Cipher.ENCRYPT_MODE, getKey());
        return cipher.doFinal(cleartext);
    }
    
    public final synchronized byte[] decrypt(byte[] ciphertext)
    throws NullPointerException, GeneralSecurityException {
        cipher.init(Cipher.DECRYPT_MODE, getKey());
        return cipher.doFinal(ciphertext);
    }
    
}
