/*
 * Local Matters, Inc. Confidential
 * OCO Source Materials
 * (C) Local Matters, Inc. 2007. All Rights Reserved.
 */
package com.localmatters.flexiq.security.store;

import com.localmatters.flexiq.security.CipherAlgorithmEnum;
import com.localmatters.flexiq.security.CipherService;
import com.localmatters.flexiq.security.CipherServiceBase;
import com.localmatters.flexiq.security.store.Passwords.Service;
import com.localmatters.flexiq.security.store.Passwords.Service.User;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import org.apache.commons.codec.binary.Base64;

/**
 * An XML file based implementation for the security store.
 *
 * @author Daniel Cavalcanti
 */
public class FileStoreService
    implements SecurityStoreService {
    
    private static final String FILE_STORE_SCHEMA = "com/localmatters/flexiq/security/store/file-store.xsd";
    private static final String FILE_STORE_TEMPLATE = "com/localmatters/flexiq/security/store/file-store-template.xml";
    
    private Unmarshaller unmarshaller;
    private Marshaller marshaller;
    private CipherService cipher;
    private Base64 codec = new Base64();
    
    private String fileName;
    private Passwords store;
    
    /**
     * Creates a new instance of FileStoreService. If file security-store does
     * not exits, an empty one from a template is created.
     *
     * @param fileName The security-store file name.
     * @param cipher The cipher used from encryption/decryption.
     * @throws javax.xml.bind.JAXBException If there is an exception in the security-store file format.
     * @throws java.io.IOException If there is an IO exception.
     */
    public FileStoreService(String fileName, CipherService cipher)
    throws JAXBException, IOException {
        
        JAXBContext context = JAXBContext.newInstance("com.localmatters.flexiq.security.store");
        unmarshaller = context.createUnmarshaller();
        marshaller = context.createMarshaller();
        
        File file = new File(fileName);
        
        if (file.exists())
            store = (Passwords) unmarshaller.unmarshal(file);
        else
            store = (Passwords) unmarshaller.unmarshal(getClass().getClassLoader().getResourceAsStream(FILE_STORE_TEMPLATE));
        
        this.fileName = fileName;
        this.cipher = cipher;
        
    }
    
    public List<String> getServiceNames() {
        
        List<Service> services = store.getService();
        List<String> serviceNames = new ArrayList<String>(services.size());
        
        for (Service service : services)
            serviceNames.add(service.getName());
        
        return serviceNames;
        
    }
    
    public boolean exist(String serviceName) {
        
        for (Service service : store.getService())
            if (serviceName.equals(service.getName()))
                return true;
        
        return false;
        
    }
    
    public void createService(String serviceName)
    throws NullPointerException, IllegalArgumentException {
        
        if (serviceName == null)
            throw new NullPointerException("Service name cannot be null.");
        if (exist(serviceName))
            throw new IllegalArgumentException("Service with service name " + serviceName + " already exists.");
        
        Passwords.Service service = new Passwords.Service();
        service.setName(serviceName);
        store.getService().add(service);
        
    }
    
    public List<String> getUsernames(String serviceName)
    throws NullPointerException, IllegalArgumentException {
        
        if (serviceName == null)
            throw new NullPointerException("Service name cannot be null.");
        
        for (Service service : store.getService()) {
            if (serviceName.equals(service.getName())) {
                List<User> users = service.getUser();
                List<String> usernames = new ArrayList<String>(users.size());
                
                for (User user : users)
                    usernames.add(user.getUsername());
                
                return usernames;
            }
        }
        
        throw new IllegalArgumentException("Service with service name " + serviceName + " does not exist.");
        
    }
    
    public void createUser(String serviceName, String username, String password)
    throws NullPointerException, IllegalArgumentException {
        
        if (serviceName == null)
            throw new NullPointerException("Service name cannot be null.");
        if (username == null)
            throw new NullPointerException("User's username cannot be null.");
        if (password == null)
            throw new NullPointerException("User's password cannot be null.");
        
        try {
            for (Service service : store.getService()) {
                if (serviceName.equals(service.getName())) {
                    
                    for (User user : service.getUser()) {
                        if (username.equals(user.getUsername()))
                            throw new IllegalArgumentException("User " + username + " already exists for service " + serviceName);
                    }
                    
                    byte[] encrypted = cipher.encrypt(password.getBytes());
                    byte[] encoded = codec.encode(encrypted);
                    
                    User user = new User();
                    user.setUsername(username);
                    user.setPassword(new String(encoded));
                    
                    service.getUser().add(user);
                    return;
                    
                }
            }
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
        
        throw new IllegalArgumentException("Service with service name " + serviceName + " does not exist.");
        
    }
    
    public String getPassword(String serviceName, String username)
    throws NullPointerException, IllegalArgumentException {
        
        if (serviceName == null)
            throw new NullPointerException("Service name cannot be null.");
        if (username == null)
            throw new NullPointerException("User's username cannot be null.");
        
        try {
            for (Service service : store.getService()) {
                if (serviceName.equals(service.getName())) {
                    for (User user : service.getUser()) {
                        if (username.equals(user.getUsername())) {
                            
                            byte[] decoded = (byte[]) codec.decode(user.getPassword().getBytes());
                            byte[] decrypted = cipher.decrypt(decoded);
                            
                            return new String(decrypted);
                            
                        }
                    }
                    throw new IllegalArgumentException("User " + username + " for service " + serviceName + " does not exist.");
                }
            }
            throw new IllegalArgumentException("Service with service name " + serviceName + " does not exist.");
        } catch (GeneralSecurityException ex) {
            throw new RuntimeException(ex);
        }
        
    }
    
    public void setPassword(String serviceName, String username, String password)
    throws NullPointerException, IllegalArgumentException {
        
        if (serviceName == null)
            throw new NullPointerException("Service name cannot be null.");
        if (username == null)
            throw new NullPointerException("User's username cannot be null.");
        if (password == null)
            throw new NullPointerException("User's password cannot be null.");
        
        try {
            for (Service service : store.getService()) {
                if (serviceName.equals(service.getName())) {
                    for (User user : service.getUser()) {
                        if (username.equals(user.getUsername())) {
                            
                            byte[] encrypted = cipher.encrypt(password.getBytes());
                            byte[] encoded = codec.encode(encrypted);
                            
                            user.setPassword(new String(encoded));
                            return;
                            
                        }
                    }
                    throw new IllegalArgumentException("User " + username + " for service " + serviceName + " does not exist.");
                }
            }
            throw new IllegalArgumentException("Service with service name " + serviceName + " does not exist.");
        } catch (GeneralSecurityException ex) {
            throw new RuntimeException(ex);
        }
        
    }
    
    public void persist() {
        try {
            marshaller.marshal(store, new File(fileName));
        } catch (JAXBException ex) {
            throw new RuntimeException(ex);
        }
    }
    
    public void close() {
    }
    
}
